UNIT RPDef;

INTERFACE

USES
    sysutils, DOM, xmlread,xmlWrite,MkBMP,OS2BASE,OS2DEF,OS2PMAPI;


TYPE
    ColorType=RECORD
        r,g,b:BYTE;
    END;
    VectorRecord=Record
        x,y,z:SINGLE;
    end;
    SpectrumRecord=record
        r,g,b:SINGLE;
    end;

    RayRecord=Record
        Org,Dir:VectorRecord;
    END;

    MaterialRecord=Record
        col:SpectrumRecord;
        reflection:single;
        refraction:single;
        RefractiveRate :single;  // ܗ
        EmissionColor:SpectrumRecord;
    END;

    // J
    CameraRecord=Record
        eye, origin, xaxis, yaxis:VectorRecord;
    end;
function CreateVector(x_,y_,z_:SINGLE):VectorRecord;
FUNCTION VecAdd(V1,V2 :VectorRecord):VectorRecord;
FUNCTION VecSub(Vo,Vs :VectorRecord):VectorRecord;
FUNCTION VecScale(V:VectorRecord;S :SINGLE):VectorRecord;
FUNCTION VecNeg(v:VectorRecord):VectorRecord;
FUNCTION Veclen(v:VectorRecord):SINGLE;
FUNCTION VecNormalize(V:VectorRecord):VectorRecord;
FUNCTION VecDot(V1,V2 :VectorRecord):SINGLE;//
FUNCTION VecCross(V1,V2 :VectorRecord):VectorRecord;//O


function CreateSpectrum(r_,g_,b_:SINGLE):SpectrumRecord;
FUNCTION SpecAdd(v1,v2:SpectrumRecord):SpectrumRecord;
FUNCTION SpecMul(v1,v2:SpectrumRecord):SpectrumRecord;
FUNCTION SpecScale(v:SpectrumRecord;s:SINGLE):SpectrumRecord;
FUNCTION SpecToColor(v:SpectrumRecord):ColorType;

function  CreateMaterial(col_:SpectrumRecord):MaterialRecord;


type
    GraphDataClass=CLASS(MingViewClass)
        Cx,Cy:integer;
        JpegQuality:INTEGER;
        isFitPicture:boolean;
        constructor Create;OverRide;
        PROCEDURE SetupHeader(px,py:INTEGER);
        FUNCTION GetDrawRect:RectL;
        PROCEDURE DrawBitmap(rect:RectL;ps:hps);
        PROCEDURE SaveFile(SaveSt:string);
        procedure SetPixel(x,y:integer;Col:ColorType);

    END;

type
  TXMLReader = CLASS
  PROTECTED
    doc: TXMLDocument;
    FileName,RootName: String;
    PrevPath,PrevAPath,PrevName:string;
    PrevNode:TDOMNode;
    CurNode:TDOMNode;
  PUBLIC
    constructor Create(CONST AFileName,ARootName: String);
    destructor Destroy; override;
    FUNCTION  GetValue(CONST APath, ADefault: String): String;
    FUNCTION  GetNextValue(CONST APath, ADefault: String): String;
    function SetFirstNode(const ACommonPath:string):TDOMNode;
    function SetNextNode:TDOMNode;
    function GetNodeValue(const APath,ADefault:string):string;
    function GetNodeVectorValue(const APath:string;
                                ADefault:VectorRecord):VectorRecord;
    function GetNodeSpectrumValue(const APath:string;
                                  ADefault:SpectrumRecord):SpectrumRecord;
  END;

var
    SkyColor,BlackColor:SpectrumRecord;

IMPLEMENTATION




function CreateVector(x_,y_,z_:SINGLE):VectorRecord;
BEGIN
    result.x:=x_;result.y:=y_;result.z:=z_;
END;
FUNCTION VecAdd(V1,V2 :VectorRecord):VectorRecord;
BEGIN
    result.x:=V1.x+V2.x;
    result.y:=V1.y+V2.y;
    result.z:=V1.z+V2.z;
END;
FUNCTION VecSub(Vo,Vs :VectorRecord):VectorRecord;
BEGIN
    result.x:=Vo.x-Vs.x;
    result.y:=Vo.y-Vs.y;
    result.z:=Vo.z-Vs.z;
END;
FUNCTION VecScale(V:VectorRecord;S :SINGLE):VectorRecord;
BEGIN
    result.x:=V.x*s;
    result.y:=V.y*s;
    result.z:=V.z*s;
END;
FUNCTION VecNeg(V:VectorRecord):VectorRecord;
BEGIN
    result.x:=-V.x;
    result.y:=-V.y;
    result.z:=-V.z;
END;
FUNCTION Veclen(V:VectorRecord):SINGLE;
BEGIN
   result:=sqrt(V.x*V.x+V.y*V.y+V.z*V.z);
END;

FUNCTION VecNormalize(V:VectorRecord):VectorRecord;
BEGIN
    result:=Vecscale(V,1.0/VecLen(V) );
END;
FUNCTION VecDot(V1,V2 :VectorRecord):SINGLE;//
BEGIN
    result:=v1.x*v2.x+v1.y*v2.y+v1.z*v2.z;
END;
FUNCTION VecCross(V1,V2 :VectorRecord):VectorRecord;//O
BEGIN
    result.x:=V1.y * v2.z - v2.y * V1.z;
    result.y:=V1.z * v2.x - v2.z * V1.x;
    result.z:=V1.x * v2.y - v2.x * V1.y;
END;


function CreateSpectrum(r_, g_, b_:SINGLE):SpectrumRecord;
BEGIN
    result.r := r_;
    result.g := g_;
    result.b := b_;
END;

FUNCTION SpecAdd(v1,v2:SpectrumRecord):SpectrumRecord;
BEGIN
    result.r:=v1.r + v2.r;
    result.g:=v1.g + v2.g;
    result.b:=v1.b + v2.b;
END;

FUNCTION SpecMul(v1,v2:SpectrumRecord):SpectrumRecord;
BEGIN
    result.r:=v1.r * v2.r;
    result.g:=v1.g * v2.g;
    result.b:=v1.b * v2.b;
END;
FUNCTION SpecScale(v:SpectrumRecord;s:SINGLE):SpectrumRecord;
BEGIN
    result.r:=v.r * s;
    result.g:=v.g * s;
    result.b:=v.b * s;
END;
FUNCTION SpecToColor(v:SpectrumRecord):ColorType;
VAR
    i:ColorType;
    FUNCTION RealToByte(r:SINGLE):BYTE;
    BEGIN
        IF r>255 THEN r:=255;
        result:=Trunc(r);
    END;
BEGIN
    i.r := RealToByte(v.r * 255);
    i.g := RealToByte(v.g * 255);
    i.b := RealToByte(v.b * 255);
    result:=i;
END;



function CreateMaterial(col_:SpectrumRecord):MaterialRecord;
begin
    result.col:=col_;
    result.reflection:=0; //˗
    result.refraction:=0; //܂̊
    result.RefractiveRate := 1;  // ܗ
    result.EmissionColor.r:=0;
    result.EmissionColor.g:=0;
    result.EmissionColor.b:=0;

end;



constructor GraphDataClass.Create;
BEGIN
    JpegQuality:=75;
    isFitPicture:=TRUE;
    inherited Create;
END;

PROCEDURE  GraphDataClass.SetupHeader(px,py:INTEGER);
begin
    inherited SetupHeader(px,py);
    Cx:=px;Cy:=py;
end;

FUNCTION GraphDataClass.GetDrawRect:RectL;
VAR
    cx,cy :LONGINT;(**bitmap size**)
    rect  :RectL;
BEGIN
    cx:=pBMPBody^.bmpCx;
    cy:=pBMPBody^.bmpCy;
    rect.xLeft:=0;Rect.xRight:=cx;
    rect.yTop:=cy;rect.yBottom:=0;
//    rect.xRight:=(Rect.xRight div 8)*8;
    GetDrawRect:=Rect;
END;


PROCEDURE GraphDataClass.DrawBitmap(rect:RectL;ps:hps);
VAR
    RectSrc,RectDest:RectL;
    hbm:HBITMAP;
    rc:bool;
    SrcXYRatio,DestXYRatio:double;
    yValue,xValue,xDiv,yDiv:integer;
BEGIN
    hbm:=GpiCreateBitmap(ps,
                        pBMPBody^.InfoHeader2,
                        CBM_INIT,
                        pBMPBody^.RG,
                        pBMPBody^.Info2);
    RectSrc.yTop:=pBMPBody^.bmpcy;
    RectSrc.yBottom:=0;
    RectSrc.xLeft:=0;
    RectSrc.xRight:=pBMPBody^.bmpCx;

    IF isFitPicture THEN BEGIN

        SrcXYRatio:=RectSrc.xRight / RectSrc.yTop;
        DestXYRatio:=rect.xRight/rect.yTop;

        RectDest:=rect;

        IF SrcXYRatio>DestXYRatio then begin
            xValue:=Rect.xRight; xDiv:=0;
            yValue:=Round(Rect.xRight / SrcXYRatio);
            yDiv:=(rect.yTop-yValue) div 2;
            RectDest.yBottom:=yDiv;
            RectDest.yTop:=RectDest.yTop-yDiv;
        end
        else begin
            yValue:=Rect.yTop; yDiv:=0;
            xValue:=Round(Rect.yTop * SrcXYRatio);
            xDiv:=(Rect.xRight-xValue) div 2;
            RectDest.xLeft:=xDiv;
            RectDest.xRight:=RectDest.xRight-xDiv;
        end;

        rc:=WinDrawBitmap(PS,Hbm,@RectSrc,@RectDest,1, 0,DBM_STRETCH);
    END
    ELSE BEGIN
        RectDest.xLeft:=( (Rect.xRight-Rect.xLeft)-
                          (RectSrc.xRight-RectSrc.xLeft) ) div 2
                          +Rect.xLeft;
        RectDest.yBottom:=( (Rect.yTop-Rect.yBottom)-
                            (RectSrc.yTop-RectSrc.yBottom) ) div 2
                          +Rect.yBottom;
        RectDest.yTop:= RectDest.yBottom+(RectSrc.yTop-RectSrc.yBottom);
        RectDest.xRight:= RectDest.xLeft+(RectSrc.xRight-RectSrc.xLeft);

        rc:=WinDrawBitmap(PS,Hbm,@RectSrc,@RectDest,1, 0,DBM_NORMAL);
    END;
    IF rc=FALSE THEN BEGIN
        HALT;
    END;
    GpiDeleteBitmap(HBM);
END;




PROCEDURE GraphDataClass.SaveFile(SaveSt:string);
BEGIN
    JpegFileSave(SaveSt,JpegQuality);
    //PingFileSave(SaveSt);
END;



procedure GraphDataClass.SetPixel(x,y:integer;Col:ColorType);
var
    Suffix:LongInt;
begin
    suffix:=(x+(Cy-y)*Cx)*3;
    pBMPBody^.rg[suffix  ]:=Col.b;
    pBMPBody^.rg[suffix+1]:=Col.g;
    pBMPBody^.rg[suffix+2]:=Col.r;

end;


constructor TXMLReader.Create(CONST AFileName,ARootName: String);
VAR
  f: File;
  cfg: TDOMElement;
BEGIN
  FileName := AFileName;
  RootName := ARootName;
  Assign(f, AFileName);
  {$I-}
  Reset(f, 1);
  {$I+}
  IF IOResult = 0 THEN BEGIN
    try
      ReadXMLFile(doc, f);
    except
      on e: EXMLReadError DO
        WRITELN(//StdErr,
                'Warning: XML config parsing error: ', e.Message);
    END;
    Close(f);
  END;

  IF NOT Assigned(doc) THEN
    doc := TXMLDocument.Create;

  cfg :=TDOMElement(doc.FindNode(RootName) );
  IF NOT Assigned(cfg) THEN BEGIN
    cfg := doc.CreateElement(RootName);
    doc.AppendChild(cfg);
  END;
  PrevPath:='';
  PrevAPath:='';
END;

destructor TXMLReader.Destroy;
BEGIN
  IF Assigned(doc) THEN
    doc.Free;
  inherited Destroy;
END;


FUNCTION TXMLReader.GetValue(CONST APath, ADefault: String): String;
VAR
  node, subnode, attr: TDOMNode;
  i: INTEGER;
  aname, path: String;
BEGIN
  node := doc.DocumentElement;
  path := APath;
  WHILE True DO BEGIN
    i := Pos('/', path);
    IF i = 0 THEN break;
    aname := Copy(path, 1, i - 1);
    path := Copy(path, i + 1, Length(path));
    subnode := node.FindNode(aname);
    IF NOT Assigned(subnode) THEN BEGIN
      Result := ADefault;
      EXIT;
    END;
    node := subnode;
  END;
  attr := node.Attributes.GetNamedItem(path);
  IF Assigned(attr) THEN
    Result := attr.NodeValue
  ELSE
    Result := ADefault;
  PrevPath:=path;
  PrevAPath:=APath;
  PrevNode:=node;
  PrevName:=aname;
END;

FUNCTION TXMLReader.GetNextValue(CONST APath, ADefault: String): String;
VAR
  subnode,node, attr: TDOMNode;
  name, path: String;
BEGIN
    IF APath=PrevAPath THEN BEGIN
        node:=PrevNode.NextSibling;
        IF NOT Assigned(node) THEN BEGIN
            result:='';
            EXIT;
        END;
        IF node.NodeName<>PrevName THEN BEGIN
            result:='';
            EXIT;
        END;
        attr := node.Attributes.GetNamedItem(PrevPath);
        IF Assigned(attr) THEN
          Result := attr.NodeValue
        ELSE
          Result := ADefault;
        PrevNode:=node;
    END
    ELSE BEGIN
        result:='';
    END;
END;


function TXMLReader.SetFirstNode(const ACommonPath:string):TDOMNode;
VAR
  node, subnode, attr: TDOMNode;
  i: INTEGER;
  aname, path: String;
BEGIN
  result:=nil;
  node := doc.DocumentElement;
  path := ACommonPath;
  WHILE True DO BEGIN
    i := Pos('/', path);
    IF i = 0 THEN break;
    aname := Copy(path, 1, i - 1);
    path := Copy(path, i + 1, Length(path));
    subnode := node.FindNode(aname);
    IF NOT Assigned(subnode) THEN BEGIN
      EXIT;
    END;
    node := subnode;
  END;
  IF NOT Assigned(subnode) THEN BEGIN
    EXIT;
  END;
  node := subnode;
  PrevPath:=aname;
  CurNode:=Node;
  result:=Node;
END;

function TXMLReader.SetNextNode:TDOMNode;
begin
    CurNode:=CurNode.NextSibling;
    while (CurNode<>Nil) and (CurNode.NodeName<>PrevPath) do
    begin
        CurNode:=CurNode.NextSibling;
    end;
    result:=CurNode;
end;


function TXMLReader.GetNodeValue(const Apath,ADefault:string):string;
VAR
  subnode,node, attr: TDOMNode;
  aname, path: String;
  i:integer;
BEGIN
    node:=CurNode;

    path:=APath;
    WHILE True DO BEGIN
      i := Pos('/', path);
      IF i = 0 THEN break;
      aname := Copy(path, 1, i - 1);
      path := Copy(path, i + 1, Length(path));
      subnode := node.FindNode(aname);
      IF NOT Assigned(subnode) THEN BEGIN
        Result := ADefault;
        EXIT;
      END;
      node := subnode;
    END;

    attr := node.Attributes.GetNamedItem(Path);
    IF Assigned(attr) THEN
      Result := attr.NodeValue
    ELSE
      Result := ADefault;
END;


function TXMLReader.GetNodeVectorValue(const APath:string;
                                ADefault:VectorRecord):VectorRecord;
begin
    result.x:=StrToFloat(GetNodeValue(APath+'/x',FloatToStr(ADefault.x))  );
    result.y:=StrToFloat(GetNodeValue(APath+'/y',FloatToStr(ADefault.y))  );
    result.z:=StrToFloat(GetNodeValue(APath+'/z',FloatToStr(ADefault.z))  );
end;

function TXMLReader.GetNodeSpectrumValue(
                                      const APath:string;
                                      ADefault:SpectrumRecord):SpectrumRecord;
begin
    result.r:=StrToFloat(GetNodeValue(APath+'/r',FloatToStr(ADefault.r))  );
    result.g:=StrToFloat(GetNodeValue(APath+'/g',FloatToStr(ADefault.g))  );
    result.b:=StrToFloat(GetNodeValue(APath+'/b',FloatToStr(ADefault.b))  );
end;



BEGIN
    BlackColor:=CreateSpectrum(0,0,0);
    SkyColor:=CreateSpectrum(0.7,0.75,0.8);
//    SkyColor:=CreateSpectrum(0.5,0.6,0.7);

END.




BEGIN
END.


$Log: rpdef.pas $
Revision 1.20  2011/06/04 17:42:41  Average
t@Cݒǂݍݔ

Revision 1.10  2011/06/04 16:06:02  Average
IVer1.10

Revision 1.2  2011/06/03 15:06:26  Average
Ƃ肠ASphereݒt@Cǂݍőn`

Revision 1.1  2011/06/03 14:23:23  Average
Initial revision

